@@ -1,20 +1,25 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# PSEXEC like functionality example using RemComSvc (https://github.com/kavika13/RemCom)
+# Description:
+#   PSEXEC like functionality example using RemComSvc (https://github.com/kavika13/RemCom)
 #
 # Author:
-#  beto (@agsolino)
+#   beto (@agsolino)
 #
 # Reference for:
-#  DCE/RPC and SMB.
+#   DCE/RPC and SMB.
+#
 
 import sys
 import os
+import re
 import cmd
 import logging
 from threading import Thread, Lock
@@ -33,6 +38,7 @@ from impacket.examples import remcomsvc, serviceinstall
 from impacket.examples.utils import parse_target
 from impacket.krb5.keytab import Keytab
 
+CODEC = sys.stdout.encoding
 
 class RemComMessage(Structure):
     structure = (
@@ -50,9 +56,9 @@ class RemComResponse(Structure):
         ('ReturnCode','<L=0'),
     )
 
-RemComSTDOUT         = "RemCom_stdout"
-RemComSTDIN          = "RemCom_stdin"
-RemComSTDERR         = "RemCom_stderr"
+RemComSTDOUT = "RemCom_stdout"
+RemComSTDIN = "RemCom_stdin"
+RemComSTDERR = "RemCom_stderr"
 
 lock = Lock()
 
@@ -79,18 +85,15 @@ class PSEXEC:
             self.__lmhash, self.__nthash = hashes.split(':')
 
     def run(self, remoteName, remoteHost):
-
         stringbinding = r'ncacn_np:%s[\pipe\svcctl]' % remoteName
         logging.debug('StringBinding %s'%stringbinding)
         rpctransport = transport.DCERPCTransportFactory(stringbinding)
         rpctransport.set_dport(self.__port)
         rpctransport.setRemoteHost(remoteHost)
-
         if hasattr(rpctransport, 'set_credentials'):
             # This method exists only for selected protocol sequences.
             rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
                                          self.__nthash, self.__aesKey)
-
         rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
         self.doStuff(rpctransport)
 
@@ -217,6 +220,7 @@ class PSEXEC:
             sys.stdout.flush()
             sys.exit(1)
 
+
 class Pipes(Thread):
     def __init__(self, transport, pipe, permissions, share=None):
         Thread.__init__(self)
@@ -262,26 +266,117 @@ class RemoteStdOutPipe(Pipes):
 
     def run(self):
         self.connectPipe()
-        while True:
-            try:
-                ans = self.server.readFile(self.tid,self.fid, 0, 1024)
-            except:
-                pass
-            else:
+
+        global LastDataSent
+
+        if PY3:
+            __stdoutOutputBuffer, __stdoutData = b"", b""
+
+            while True:
                 try:
-                    global LastDataSent
-                    if ans != LastDataSent:
-                        sys.stdout.write(ans.decode('cp437'))
-                        sys.stdout.flush()
-                    else:
-                        # Don't echo what I sent, and clear it up
-                        LastDataSent = ''
-                    # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
-                    # it will give false positives tho.. we should find a better way to handle this.
-                    if LastDataSent > 10:
-                        LastDataSent = ''
+                    stdout_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
                 except:
                     pass
+                else:
+                    try:
+                        if stdout_ans != LastDataSent:
+                            if len(stdout_ans) != 0:
+                                # Append new data to the buffer while there is data to read
+                                __stdoutOutputBuffer += stdout_ans
+
+                        promptRegex = b'([a-zA-Z]:[\\\/])((([a-zA-Z0-9 -\.]*)[\\\/]?)+(([a-zA-Z0-9 -\.]+))?)?>$'
+
+                        endsWithPrompt = bool(re.match(promptRegex, __stdoutOutputBuffer) is not None)
+                        if endsWithPrompt == True:
+                            # All data, we shouldn't have encoding errors
+                            # Adding a space after the prompt because it's beautiful
+                            __stdoutData = __stdoutOutputBuffer + b" "
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = b""
+
+                            # print("[+] endsWithPrompt")
+                            # print(" | __stdoutData:",__stdoutData)
+                            # print(" | __stdoutOutputBuffer:",__stdoutOutputBuffer)
+                        elif b'\n' in __stdoutOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stdoutOutputBuffer.split(b"\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stdoutData = b"\n".join(lines[:-1]) + b"\n"
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = lines[-1]
+                            # print("[+] newline in __stdoutOutputBuffer")
+                            # print(" | __stdoutData:",__stdoutData)
+                            # print(" | __stdoutOutputBuffer:",__stdoutOutputBuffer)
+
+                        if len(__stdoutData) != 0:
+                            # There is data to print
+                            try:
+                                sys.stdout.write(__stdoutData.decode(CODEC))
+                                sys.stdout.flush()
+                                __stdoutData = b""
+                            except UnicodeDecodeError:
+                                logging.error('Decoding error detected, consider running chcp.com at the target,\nmap the result with '
+                                              'https://docs.python.org/3/library/codecs.html#standard-encodings\nand then execute smbexec.py '
+                                              'again with -codec and the corresponding codec')
+                                print(__stdoutData.decode(CODEC, errors='replace'))
+                                __stdoutData = b""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = b""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except:
+                        pass
+        else:
+            __stdoutOutputBuffer, __stdoutData = "", ""
+
+            while True:
+                try:
+                    stdout_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
+                except:
+                    pass
+                else:
+                    try:
+                        if stdout_ans != LastDataSent:
+                            if len(stdout_ans) != 0:
+                                # Append new data to the buffer while there is data to read
+                                __stdoutOutputBuffer += stdout_ans
+
+                        promptRegex = r'([a-zA-Z]:[\\\/])((([a-zA-Z0-9 -\.]*)[\\\/]?)+(([a-zA-Z0-9 -\.]+))?)?>$'
+
+                        endsWithPrompt = bool(re.match(promptRegex, __stdoutOutputBuffer) is not None)
+                        if endsWithPrompt:
+                            # All data, we shouldn't have encoding errors
+                            # Adding a space after the prompt because it's beautiful
+                            __stdoutData = __stdoutOutputBuffer + " "
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = ""
+
+                        elif '\n' in __stdoutOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stdoutOutputBuffer.split("\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stdoutData = "\n".join(lines[:-1]) + "\n"
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = lines[-1]
+
+                        if len(__stdoutData) != 0:
+                            # There is data to print
+                            sys.stdout.write(__stdoutData.decode(CODEC))
+                            sys.stdout.flush()
+                            __stdoutData = ""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = ""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except Exception as e:
+                        pass
+
 
 class RemoteStdErrPipe(Pipes):
     def __init__(self, transport, pipe, permisssions):
@@ -289,17 +384,87 @@ class RemoteStdErrPipe(Pipes):
 
     def run(self):
         self.connectPipe()
-        while True:
-            try:
-                ans = self.server.readFile(self.tid,self.fid, 0, 1024)
-            except:
-                pass
-            else:
+
+        if PY3:
+            __stderrOutputBuffer, __stderrData = b'', b''
+
+            while True:
                 try:
-                    sys.stderr.write(str(ans))
-                    sys.stderr.flush()
+                    stderr_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
                 except:
                     pass
+                else:
+                    try:
+                        if len(stderr_ans) != 0:
+                            # Append new data to the buffer while there is data to read
+                            __stderrOutputBuffer += stderr_ans
+
+                        if b'\n' in __stderrOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stderrOutputBuffer.split(b"\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stderrData = b"\n".join(lines[:-1]) + b"\n"
+                            # Remainder data for next iteration
+                            __stderrOutputBuffer = lines[-1]
+
+                        if len(__stderrData) != 0:
+                            # There is data to print
+                            try:
+                                sys.stdout.write(__stderrData.decode(CODEC))
+                                sys.stdout.flush()
+                                __stderrData = b""
+                            except UnicodeDecodeError:
+                                logging.error('Decoding error detected, consider running chcp.com at the target,\nmap the result with '
+                                              'https://docs.python.org/3/library/codecs.html#standard-encodings\nand then execute smbexec.py '
+                                              'again with -codec and the corresponding codec')
+                                print(__stderrData.decode(CODEC, errors='replace'))
+                                __stderrData = b""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = b""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except Exception as e:
+                        pass
+        else:
+            __stderrOutputBuffer, __stderrData = '', ''
+
+            while True:
+                try:
+                    stderr_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
+                except:
+                    pass
+                else:
+                    try:
+                        if len(stderr_ans) != 0:
+                            # Append new data to the buffer while there is data to read
+                            __stderrOutputBuffer += stderr_ans
+
+                        if '\n' in __stderrOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stderrOutputBuffer.split("\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stderrData = "\n".join(lines[:-1]) + "\n"
+                            # Remainder data for next iteration
+                            __stderrOutputBuffer = lines[-1]
+
+                        if len(__stderrData) != 0:
+                            # There is data to print
+                            sys.stdout.write(__stderrData.decode(CODEC))
+                            sys.stdout.flush()
+                            __stderrData = ""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = ""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except:
+                        pass
+
 
 class RemoteShell(cmd.Cmd):
     def __init__(self, server, port, credentials, tid, fid, share, transport):
@@ -398,9 +563,9 @@ class RemoteShell(cmd.Cmd):
 
     def default(self, line):
         if PY3:
-            self.send_data(line.encode('cp437')+b'\r\n')
+            self.send_data(line.encode(CODEC)+b'\r\n')
         else:
-            self.send_data(line.decode(sys.stdin.encoding).encode('cp437')+'\r\n')
+            self.send_data(line.decode(sys.stdin.encoding).encode(CODEC)+'\r\n')
 
     def send_data(self, data, hideOutput = True):
         if hideOutput is True:
@@ -435,6 +600,11 @@ if __name__ == '__main__':
     parser.add_argument('-file', action='store', help="alternative RemCom binary (be sure it doesn't require CRT)")
     parser.add_argument('-ts', action='store_true', help='adds timestamp to every logging output')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+    parser.add_argument('-codec', action='store', help='Sets encoding used (codec) from the target\'s output (default '
+                                                       '"%s"). If errors are detected, run chcp.com at the target, '
+                                                       'map the result with '
+                          'https://docs.python.org/3/library/codecs.html#standard-encodings and then execute smbexec.py '
+                          'again with -codec and the corresponding codec ' % CODEC)
 
     group = parser.add_argument_group('authentication')
 
@@ -471,6 +641,12 @@ if __name__ == '__main__':
     # Init the example's logger theme
     logger.init(options.ts)
 
+    if options.codec is not None:
+        CODEC = options.codec
+    else:
+        if CODEC is None:
+            CODEC = 'utf-8'
+
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
         # Print the Library's installation path
